/*
 * Copyright (c) 2010-2012 OTClient <https://github.com/edubart/otclient>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "eventdispatcher.h"

#include <framework/core/clock.h>

EventDispatcher g_dispatcher;

void EventDispatcher::flush()
{
    poll();

    while(!m_scheduledEventList.empty())
        m_scheduledEventList.pop();
}

void EventDispatcher::poll()
{
    while(!m_scheduledEventList.empty()) {
        if(g_clock.ticks() < m_scheduledEventList.top().ticks)
            break;
        SimpleCallback callback = std::move(m_scheduledEventList.top().callback);
        m_scheduledEventList.pop();
        callback();
    }

    while(!m_eventList.empty()) {
        m_eventList.front()();
        m_eventList.pop_front();
    }
}

void EventDispatcher::scheduleEvent(const SimpleCallback& callback, int delay)
{
    assert(delay >= 0);
    m_scheduledEventList.push(ScheduledEvent(g_clock.ticksFor(delay), callback));
}

void EventDispatcher::addEvent(const SimpleCallback& callback, bool pushFront)
{
    if(pushFront)
        m_eventList.push_front(callback);
    else
        m_eventList.push_back(callback);
}
